unit iScript;

interface

uses
    Windows, SysUtils, Classes, Variants, ComObj, ActiveX, Dialogs;

type
    TScriptError = record
        Number: integer;
        Source: string;
        Description: string;
        HelpFile: string;
        HelpContext: integer;
        Text: string;
        Line: integer;
        Column: integer;
    end;

    TScriptControl = class
    private
        FSCRIPT: Variant;
    public
        constructor Create();
        destructor Destroy(); override;
        procedure SetTimeOut(const Value: integer);

        function ProcCount(): integer;
        procedure ProcNames(Names: TStrings);

        procedure AddObject(const ProcName, ObjectName: string; const AddMembers: Boolean = true);
        procedure AddCode(const Code: string);
        function Eval(const Expression: string): Variant;
        procedure ExecuteStatement(const Statement: string);
        procedure Reset();
        procedure Run(const ProcName: string);

        function Error(): TScriptError;
        procedure AbountBox();

        procedure Test();
    end;

var
    Scripter: TScriptControl = nil;

implementation

//-----------------------------------------------------------------------------
//  RXgN^
constructor TScriptControl.Create();
begin
    FSCRIPT := CreateOleObject('ScriptControl');
    FSCRIPT.Language := 'VBScript';
end;

//-----------------------------------------------------------------------------
//  fXgN^
destructor TScriptControl.Destroy();
begin
    FSCRIPT := Unassigned;
end;

//-----------------------------------------------------------------------------
//  ^CAEgݒ
procedure TScriptControl.SetTimeOut(const Value: integer);
begin
    FSCRIPT.Timeout := Value;
end;

//-----------------------------------------------------------------------------
//  vV[W̐
function TScriptControl.ProcCount(): integer;
begin
    Result := FScript.Procedures.Count;
end;

//-----------------------------------------------------------------------------
//  vV[W̖O
procedure TScriptControl.ProcNames(Names: TStrings);
const
    PROC_TYPE: array[Boolean] of string = ('Sub', 'Function');
var
    i: integer;
    Name: string;
    ArgNum: integer;
    IsFunc: Boolean;
begin
    Names.Clear();
    for i := 1 to ProcCount do
    begin
        Name   := FScript.Procedures.Item[i].Name;
        ArgNum := FScript.Procedures.Item[i].NumArgs;
        IsFunc := FScript.Procedures.Item[i].HasReturnValue;
        Names.Add(Format('%s %s(̐=%d)', [PROC_TYPE[IsFunc], Name, ArgNum]));
    end;
end;

//-----------------------------------------------------------------------------
//  IuWFNgǉ
procedure TScriptControl.AddObject(const ProcName, ObjectName: string; const AddMembers: Boolean = true);
begin
    //  RĒN񂾂낤H
    //  Ăяoō点悢
    FSCRIPT.AddObject(ProcName, CreateOleObject(ObjectName), AddMembers);
end;

//-----------------------------------------------------------------------------
//  R[hǉ
procedure TScriptControl.AddCode(const Code: string);
begin
    FSCRIPT.AddCode(Code);
end;

//-----------------------------------------------------------------------------
//  ̕]
function TScriptControl.Eval(const Expression: string): Variant;
begin
    Result := FSCRIPT.Eval(Expression);
end;

//-----------------------------------------------------------------------------
//  ̎s
procedure TScriptControl.ExecuteStatement(const Statement: string);
begin
    FSCRIPT.ExecuteStatement(Statement);
end;

//-----------------------------------------------------------------------------
//  Zbg(H)
procedure TScriptControl.Reset();
begin
    FSCRIPT.Reset;
end;

//-----------------------------------------------------------------------------
//  s
procedure TScriptControl.Run(const ProcName: string);
begin
    FSCRIPT.Run(ProcName);
end;

//-----------------------------------------------------------------------------
//  G[擾
function TScriptControl.Error(): TScriptError;
begin
    Result.Number      := FSCRIPT.Error.Number;
    Result.Source      := FSCRIPT.Error.Source;
    Result.Description := FSCRIPT.Error.Description;
    Result.HelpFile    := FSCRIPT.Error.HelpFile;
    Result.HelpContext := FSCRIPT.Error.HelpContext;
    Result.Text        := FSCRIPT.Error.Text;
    Result.Line        := FSCRIPT.Error.Line;
    Result.Column      := FSCRIPT.Error.Column;
end;

//-----------------------------------------------------------------------------
//  AbountBox
procedure TScriptControl.AbountBox();
begin
    FSCRIPT._AboutBox;
end;

//-----------------------------------------------------------------------------
//  eXgeXgeXgeXgeXg
procedure TScriptControl.Test();
var
    i: integer;
begin
//    for i := 1 to FScript.Modules.Count do
//    begin
//        ShowMessage(FScript.Modules.Item[i].Name);
//    end;
end;

var
//    Test: TStringList;
//    E: TScriptError;
    i: integer;

initialization
    CoInitialize(nil);
    Scripter := TScriptControl.Create;

    //  ^CAEgݒ肵Ă݂eXg(ۂɂǂ͕sG[ɂ͂ȂȂ)
//    Scripter.SetTimeOut(1000);
    //  R[hǉĂ݂eXg
    Scripter.AddCode('sub main'#13#10'Call Msg3Box("Hello World")'#13#10'end sub');
    //  ]Ă݂eXg
//    ShowMessage(VarToStr(Scripter.Eval('sin(100+200)')));
    //  ǉς݂̃vV[Ẅꗗ\Ă݂eXg
//    Test := TStringList.Create;
//    Scripter.ProcNames(Test);
//    ShowMessage(Test.Text);
//    Test.Free;
    //  AoEg{bNX\Ă݂eXg
//    Scripter.AbountBox;
    //  IuWFNgǉĂ݂eXg
    Scripter.AddObject('FileSystem', 'Scripting.FileSystemObject');
    i := Scripter.Eval('FileSystem.Drives.Count');
    ShowMessage(Format('you have %d drives', [i]));
    //  p̃eXg
    Scripter.Test;
    //  G[擾Ă݂eXg
//    try
//        Scripter.Run('Main');
//    except
//        E := Scripter.Error();
//        ShowMessage(E.Description);
//    end;

finalization
    Scripter.Free;
    CoUninitialize();
end.

